@@ -921,6 +921,8 @@ class ShipmentOut(ModelWorkflow, ModelSQL, ModelView):
                     'planned_date': move.planned_date,
                     'state': 'draft',
                     'company': move.company.id,
+                    'currency': move.currency.id,
+                    'unit_price': move.unit_price,
                     })
 
     def create(self, values):
